@@ -302,6 +302,19 @@ gnome_print_dialog_construct (GnomePrintDialog *gpd, const guchar *title, gint f
 		l = gtk_label_new_with_mnemonic (_("Job"));
 		gtk_widget_show (l);
 		gtk_notebook_append_page (GTK_NOTEBOOK (gpd->notebook), gpd->job, l);
+
+		{
+			int copies;
+			gboolean collate;
+
+			if (!gnome_print_config_get_int (gpd->config, 
+			        GNOME_PRINT_KEY_NUM_COPIES, &copies))
+				copies = 1;
+			if (!gnome_print_config_get_boolean (gpd->config, 
+				GNOME_PRINT_KEY_COLLATE, &collate))
+				collate = FALSE;
+			gnome_print_dialog_set_copies (gpd, copies, collate);
+		}
 		
 		/* Add the printers page */
 		hb = gtk_hbox_new (FALSE, 0);
