@@ -32,7 +32,9 @@
 #include <libgnomeprint/gnome-print-paper.h>
 #include <libgnomeprint/gnome-print-config.h>
 #include <libgnomeprint/private/gpa-node.h>
+#include <libgnomeprint/private/gpa-utils.h>
 #include <libgnomeprint/private/gnome-print-private.h>
+#include <libgnomeprint/private/gnome-print-config-private.h>
 
 #include "gnome-print-i18n.h"
 #include "gnome-print-paper-selector.h"
@@ -56,7 +58,7 @@
 struct _GnomePaperSelector {
 	GtkHBox box;
 
-	guint blocked : 1;
+	guint updating : 1;
 
 	GnomePrintConfig *config;
 	gint flags;
@@ -177,6 +179,9 @@ gps_psize_value_changed (GtkAdjustment *adj, GnomePaperSelector *ps)
 	const GnomePrintUnit *unit;
 	gdouble w, h, max_wh;
 
+	if (ps->updating)
+		return;
+
 	unit = gnome_print_unit_selector_get_unit (GNOME_PRINT_UNIT_SELECTOR (ps->us));
 	w = gtk_spin_button_get_value (GTK_SPIN_BUTTON (ps->pw));
 
@@ -213,42 +218,36 @@ gps_m_size_value_changed (GtkAdjustment *adj, GnomePaperSelector *ps)
 	gdouble mt, mb, ml, mr;
 
 	unit = gnome_print_unit_selector_get_unit (GNOME_PRINT_UNIT_SELECTOR (ps->us));
+
 	mt = gtk_spin_button_get_value (GTK_SPIN_BUTTON (ps->margin_top));
-#ifdef GPP_VERBOSE
-	g_print ("Top Margin %g\n", mt);
-#endif
 	gnome_print_convert_distance (&mt, unit, GNOME_PRINT_PS_UNIT);
 	gnome_print_config_set_length (ps->config, GNOME_PRINT_KEY_PAGE_MARGIN_TOP, mt, GNOME_PRINT_PS_UNIT);
 
 	mb = gtk_spin_button_get_value (GTK_SPIN_BUTTON (ps->margin_bottom));
-#ifdef GPP_VERBOSE
-	g_print ("Bottom Margin %g\n", mb);
-#endif
 	gnome_print_convert_distance (&mb, unit, GNOME_PRINT_PS_UNIT);
 	gnome_print_config_set_length (ps->config, GNOME_PRINT_KEY_PAGE_MARGIN_BOTTOM, mb, GNOME_PRINT_PS_UNIT);
 
 	ml = gtk_spin_button_get_value (GTK_SPIN_BUTTON (ps->margin_left));
-#ifdef GPP_VERBOSE
-	g_print ("Bottom Margin %g\n", ml);
-#endif
 	gnome_print_convert_distance (&ml, unit, GNOME_PRINT_PS_UNIT);
 	gnome_print_config_set_length (ps->config, GNOME_PRINT_KEY_PAGE_MARGIN_LEFT, ml, GNOME_PRINT_PS_UNIT);
 
 	mr = gtk_spin_button_get_value (GTK_SPIN_BUTTON (ps->margin_right));
-#ifdef GPP_VERBOSE
-	g_print ("Bottom Margin %g\n", mr);
-#endif
 	gnome_print_convert_distance (&mr, unit, GNOME_PRINT_PS_UNIT);
 	gnome_print_config_set_length (ps->config, GNOME_PRINT_KEY_PAGE_MARGIN_RIGHT, mr, GNOME_PRINT_PS_UNIT);
 
 	if ((fabs (ps->mt - mt) < 0.1) && (fabs (ps->mb - mb) < 0.1) &&
-	    (fabs (ps->ml - ml) < 0.1) && (fabs (ps->mr - mr) < 0.1)) return;
+	    (fabs (ps->ml - ml) < 0.1) && (fabs (ps->mr - mr) < 0.1))
+		return;
 
 	ps->ml = ml;
 	ps->mr = mr;
 	ps->mt = mt;
 	ps->mb = mb;
 
+	gpa_paper_preview_item_set_logical_margins 
+		(GPA_PAPER_PREVIEW_ITEM (GNOME_PAPER_PREVIEW 
+					 (ps->preview)->item),
+		 ml, mr, mt, mb);
 }
 
 static void
@@ -274,6 +273,7 @@ gnome_paper_selector_init (GnomePaperSelector *ps)
 {
 	ps->config = NULL;
 	ps->flags = 0;
+	ps->updating = FALSE;
 }
 
 static void
@@ -436,36 +436,58 @@ static void
 gnome_paper_unit_selector_request_update_cb (GPANode *node, guint flags,  GnomePaperSelector *ps)
 {
 	guchar *unit_txt;
+
+	if (ps->updating)
+		return;
+
 	unit_txt = gnome_print_config_get (ps->config, GNOME_PRINT_KEY_PREFERED_UNIT);
-	if (unit_txt) {
-		gnome_print_unit_selector_set_unit (GNOME_PRINT_UNIT_SELECTOR (ps->us),
-						    gnome_print_unit_get_by_abbreviation
-						    (unit_txt));
-		g_free (unit_txt);
+	if (!unit_txt) {
+		g_warning ("Could not get GNOME_PRINT_KEY_PREFERED_UNIT");
+		return;
 	}
+
+	gnome_print_unit_selector_set_unit (GNOME_PRINT_UNIT_SELECTOR (ps->us),
+					    gnome_print_unit_get_by_abbreviation
+					    (unit_txt));
+	g_free (unit_txt);
 }
 
 /**
- * gnome_paper_selector_paper_size_sensitivity_update:
+ * gnome_paper_selector_paper_size_modified_cb
  * @node: 
  * @flags: 
  * @ps: 
  * 
- * Updates the sensitivity of the paper size widgets used for Custom sizes
+ * Handle paper sizes changes
  **/
 static void
-gnome_paper_selector_paper_size_sensitivity_update (GPANode *node, guint flags, GnomePaperSelector *ps)
+gnome_paper_selector_paper_size_modified_cb (GPANode *node, guint flags, GnomePaperSelector *ps)
 {
+	const GnomePrintUnit *unit, *to;
 	guchar *id;
+	gboolean sensitivity = FALSE;
+	gdouble height, width;
+	gint retval = 0;
 
 	id = gnome_print_config_get (ps->config, GNOME_PRINT_KEY_PAPER_SIZE);
-	if (id && *id && !strcmp (id, "Custom")) {
-		gtk_widget_set_sensitive (ps->pw, TRUE);
-		gtk_widget_set_sensitive (ps->ph, TRUE);
-	} else {
-		gtk_widget_set_sensitive (ps->pw, FALSE);
-		gtk_widget_set_sensitive (ps->ph, FALSE);
-	}
+	if (id && *id && !strcmp (id, "Custom"))
+		sensitivity = TRUE;
+	
+	gtk_widget_set_sensitive (ps->pw, sensitivity);
+	gtk_widget_set_sensitive (ps->ph, sensitivity);
+
+	to = gnome_print_unit_selector_get_unit (GNOME_PRINT_UNIT_SELECTOR (ps->us));
+	height = 0.0;
+	width  = 0.0;
+	retval += gnome_print_config_get_length (ps->config, GNOME_PRINT_KEY_PAPER_HEIGHT, &height, &unit);
+	retval += gnome_print_config_get_length (ps->config, GNOME_PRINT_KEY_PAPER_WIDTH,  &width,  &unit);
+	g_return_if_fail (retval == 2 && to != NULL);
+	gnome_print_convert_distance (&width,  unit, to);
+	gnome_print_convert_distance (&height, unit, to);
+	ps->updating = TRUE;
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (ps->ph), height);
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (ps->pw), width);
+	ps->updating = FALSE;
 	if (id)
 		g_free (id);
 }
@@ -489,10 +511,12 @@ gnome_paper_selector_hook_paper_size (GnomePaperSelector *ps)
 	if (ps->paper_size_node) {
 		ps->handler_paper_size = g_signal_connect (
 			G_OBJECT (ps->paper_size_node), "modified",
-			(GCallback) gnome_paper_selector_paper_size_sensitivity_update, ps);
+			(GCallback) gnome_paper_selector_paper_size_modified_cb, ps);
+	} else {
+		g_print ("No paper size node\n");
 	}
 	
-	gnome_paper_selector_paper_size_sensitivity_update (NULL, 0, ps);
+	gnome_paper_selector_paper_size_modified_cb (NULL, 0, ps);
 }
 
 static void
@@ -545,7 +569,7 @@ gnome_paper_selector_construct (GnomePaperSelector *ps)
 	gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
 	gtk_table_attach_defaults (GTK_TABLE (t), l, 0, 1, 0, 1);
 
-	ps->pmenu = gpa_option_menu_new (ps->config, GNOME_PRINT_KEY_PAPER_SIZE, printer);
+	ps->pmenu = gpa_option_menu_new (ps->config, GNOME_PRINT_KEY_PAPER_SIZE);
 	gtk_widget_show (ps->pmenu);
 	gtk_table_attach_defaults (GTK_TABLE (t), ps->pmenu, 1, 4, 0, 1);
 	gtk_label_set_mnemonic_widget ((GtkLabel *) l, ps->pmenu);
@@ -563,6 +587,7 @@ gnome_paper_selector_construct (GnomePaperSelector *ps)
 	ps->w = config_width;
 	wa = gtk_adjustment_new (config_width, 0.0001, 10000, 1, 10, 10);
 	ps->pw = gtk_spin_button_new (GTK_ADJUSTMENT (wa), 1, 2);
+	gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (ps->pw), TRUE);
 	gtk_widget_show (ps->pw);
 	gtk_table_attach_defaults (GTK_TABLE (t), ps->pw, 2, 3, 1, 2);
 	gtk_label_set_mnemonic_widget ((GtkLabel *) l, ps->pw);
@@ -580,6 +605,7 @@ gnome_paper_selector_construct (GnomePaperSelector *ps)
 	ps->h = config_height;
 	ha = gtk_adjustment_new (config_height, 0.0001, 10000, 1, 10, 10);
 	ps->ph = gtk_spin_button_new (GTK_ADJUSTMENT (ha), 1, 2);
+	gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (ps->ph), TRUE);
 	gtk_widget_show (ps->ph);
 	gtk_table_attach_defaults (GTK_TABLE (t), ps->ph, 2, 3, 2, 3);
 	gtk_label_set_mnemonic_widget ((GtkLabel *) l, ps->ph);
@@ -601,7 +627,7 @@ gnome_paper_selector_construct (GnomePaperSelector *ps)
 	gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
 	gtk_table_attach_defaults (GTK_TABLE (t), l, 0, 1, 3, 4);
 
-	ps->pomenu = gpa_option_menu_new (ps->config, GNOME_PRINT_KEY_PAPER_ORIENTATION, printer);
+	ps->pomenu = gpa_option_menu_new (ps->config, GNOME_PRINT_KEY_PAPER_ORIENTATION);
 	gtk_table_attach_defaults (GTK_TABLE (t), ps->pomenu, 1, 4, 3, 4);
 	gtk_label_set_mnemonic_widget ((GtkLabel *) l, ps->pomenu);
 	gps_set_labelled_by_relation (ps->pomenu, l);
@@ -617,7 +643,7 @@ gnome_paper_selector_construct (GnomePaperSelector *ps)
 	gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
 	gtk_table_attach_defaults (GTK_TABLE (t), l, 0, 1, 4, 5);
 
-	ps->lomenu = gpa_option_menu_new (ps->config, GNOME_PRINT_KEY_PAGE_ORIENTATION, printer);
+	ps->lomenu = gpa_option_menu_new (ps->config, GNOME_PRINT_KEY_PAGE_ORIENTATION);
 	gtk_widget_show (ps->lomenu);
 	gtk_table_attach_defaults (GTK_TABLE (t), ps->lomenu, 1, 4, 4, 5);
 	gtk_label_set_mnemonic_widget ((GtkLabel *) l, ps->lomenu);
@@ -629,7 +655,7 @@ gnome_paper_selector_construct (GnomePaperSelector *ps)
 	gtk_misc_set_alignment (GTK_MISC (l), 1.0, 0.5);
 	gtk_table_attach_defaults (GTK_TABLE (t), l, 0, 1, 5, 6);
 
-	ps->lymenu = gpa_option_menu_new (ps->config, GNOME_PRINT_KEY_LAYOUT, printer);
+	ps->lymenu = gpa_option_menu_new (ps->config, GNOME_PRINT_KEY_LAYOUT);
 	gtk_widget_show (ps->lymenu);
 	gtk_table_attach_defaults (GTK_TABLE (t), ps->lymenu, 1, 4, 5, 6);
 	gtk_label_set_mnemonic_widget ((GtkLabel *) l, ps->lymenu);
